@@ -53,6 +53,7 @@
 #include <libxml/tree.h>
 
 #include "types.h"
+#include "dmilog.h"
 
 #define MAXVAL 1024
 
@@ -111,14 +112,6 @@ static const dmi_codes_major dmiCodesMajor[] = {
         {-1, NULL, NULL, NULL}
 };
 
-typedef struct _dmi_minor {
-        long id;
-        dmi_codes_major *major;
-        char *key;
-        char value[MAXVAL];
-        struct _dmi_minor *next;
-} dmi_minor;
-
 /*** dmiopt.h ***/
 typedef struct _options {
         const char *devmem;
@@ -128,6 +121,7 @@ typedef struct _options {
         char *python_xml_map;
         xmlNode *dmiversion_n;
         char *dumpfile;
+        Log_t *logdata;
 } options;
 
 #endif
